from typing import Any, final

import numpy as np
import pandas as pd
import polars as pl
import pyarrow as pa

import vortex as vx

class Array:
    def __lt__(self, other: Array) -> Array: ...
    def __le__(self, other: Array) -> Array: ...
    def __gt__(self, other: Array) -> Array: ...
    def __ge__(self, other: Array) -> Array: ...
    def __len__(self) -> int: ...
    @staticmethod
    def from_arrow(obj: pa.Array | pa.ChunkedArray | pa.Table) -> Array: ...
    def to_arrow_array(self) -> pa.Array: ...
    @property
    def encoding(self) -> str: ...
    @property
    def nbytes(self) -> int: ...
    @property
    def dtype(self) -> vx.DType: ...
    def filter(self, mask: Array) -> Array: ...
    def fill_forward(self) -> Array: ...
    def scalar_at(self, index: int) -> Any: ...
    def take(self, indices: Array) -> Array: ...
    def slice(self, start: int, end: int) -> Array: ...
    def tree_display(self) -> str: ...
    def to_arrow_table(self) -> pa.Table: ...
    def to_numpy(self, *, zero_copy_only: bool = True) -> np.ndarray: ...
    def to_pandas_df(self) -> pd.DataFrame: ...
    def to_polars_dataframe(self) -> pl.DataFrame: ...
    def to_polars_series(self) -> pl.Series: ...
    def to_pylist(self) -> list: ...

@final
class NullTypeArray(Array): ...

@final
class BoolTypeArray(Array): ...

@final
class UInt8TypeArray(Array): ...

@final
class UInt16TypeArray(Array): ...

@final
class UInt32TypeArray(Array): ...

@final
class UInt64TypeArray(Array): ...

@final
class Int8TypeArray(Array): ...

@final
class Int16TypeArray(Array): ...

@final
class Int32TypeArray(Array): ...

@final
class Int64TypeArray(Array): ...

@final
class Float16TypeArray(Array): ...

@final
class Float32TypeArray(Array): ...

@final
class Float64TypeArray(Array): ...

@final
class Utf8TypeArray(Array): ...

@final
class BinaryTypeArray(Array): ...

@final
class StructTypeArray(Array): ...

@final
class ListTypeArray(Array): ...

@final
class ExtensionTypeArray(Array): ...

@final
class NullArray(Array): ...

@final
class BoolArray(Array): ...

@final
class PrimitiveArray(Array): ...

@final
class VarBinArray(Array): ...

@final
class VarBinViewArray(Array): ...

@final
class StructArray(Array):
    def field(self, name: str) -> Array: ...

@final
class ListArray(Array): ...

@final
class ExtensionArray(Array): ...

@final
class ConstantArray(Array):
    def scalar(self) -> vx.Scalar: ...

@final
class ChunkedArray(Array):
    def chunks(self) -> list[Array]: ...

@final
class AlpArray(Array): ...

@final
class AlpRdArray(Array): ...

@final
class DateTimePartsArray(Array): ...

@final
class DictArray(Array): ...

@final
class FsstArray(Array): ...

@final
class RunEndArray(Array): ...

@final
class SparseArray(Array): ...

@final
class ZigZagArray(Array): ...

@final
class FastLanesBitPackedArray(Array): ...

@final
class FastLanesDeltaArray(Array): ...

@final
class FastLanesForArray(Array): ...
